@@ -1,3 +1,4 @@
+import sys
 from typing import Dict, Any
 
 from typing_extensions import Final
@@ -28,6 +29,25 @@ TOP_LEVEL_NAME = '__top_level__'  # type: Final # Special function representing
 # Maximal number of subclasses for a class to trigger fast path in isinstance() checks.
 FAST_ISINSTANCE_MAX_SUBCLASSES = 2  # type: Final
 
+IS_32_BIT_PLATFORM = sys.maxsize < (1 << 31)  # type: Final
+
+PLATFORM_SIZE = 4 if IS_32_BIT_PLATFORM else 8
+
+# Runtime C library files
+RUNTIME_C_FILES = [
+    'init.c',
+    'getargs.c',
+    'int_ops.c',
+    'list_ops.c',
+    'dict_ops.c',
+    'str_ops.c',
+    'set_ops.c',
+    'tuple_ops.c',
+    'exc_ops.c',
+    'misc_ops.c',
+    'generic_ops.c',
+]  # type: Final
+
 
 def decorator_helper_name(func_name: str) -> str:
     return '__mypyc_{}_decorator_helper__'.format(func_name)
