@@ -13,23 +13,22 @@ from mypy.nodes import (
 )
 from mypyc.ir.ops import (
     Value, BasicBlock, LoadInt, Branch, Register, AssignmentTarget, TupleGet,
-    AssignmentTargetTuple, TupleSet, OpDescription
+    AssignmentTargetTuple, TupleSet, BinaryIntOp
 )
 from mypyc.ir.rtypes import (
     RType, is_short_int_rprimitive, is_list_rprimitive, is_sequence_rprimitive,
-    RTuple, is_dict_rprimitive
+    RTuple, is_dict_rprimitive, short_int_rprimitive
 )
+from mypyc.primitives.registry import CFunctionDescription
 from mypyc.primitives.dict_ops import (
     dict_next_key_op, dict_next_value_op, dict_next_item_op, dict_check_size_op,
     dict_key_iter_op, dict_value_iter_op, dict_item_iter_op
 )
-from mypyc.primitives.int_ops import unsafe_short_add
 from mypyc.primitives.list_ops import new_list_op, list_append_op, list_get_item_unsafe_op
 from mypyc.primitives.generic_ops import iter_op, next_op
 from mypyc.primitives.exc_ops import no_err_occurred_op
 from mypyc.irbuild.builder import IRBuilder
 
-
 GenFunc = Callable[[], None]
 
 
@@ -93,7 +92,7 @@ def translate_list_comprehension(builder: IRBuilder, gen: GeneratorExpr) -> Valu
 
     def gen_inner_stmts() -> None:
         e = builder.accept(gen.left_expr)
-        builder.primitive_op(list_append_op, [list_ops, e], gen.line)
+        builder.call_c(list_append_op, [list_ops, e], gen.line)
 
     comprehension_helper(builder, loop_params, gen_inner_stmts, gen.line)
     return list_ops
@@ -243,7 +242,7 @@ def make_for_loop_generator(builder: IRBuilder,
         if (expr.callee.fullname == 'builtins.reversed'
                 and len(expr.args) == 1
                 and expr.arg_kinds == [ARG_POS]
-                and is_sequence_rprimitive(rtyp)):
+                and is_sequence_rprimitive(builder.node_type(expr.args[0]))):
             # Special case "for x in reversed(<list>)".
             expr_reg = builder.accept(expr.args[0])
             target_type = builder.get_sequence_type(expr)
@@ -333,11 +332,7 @@ class ForGenerator:
 
     def load_len(self, expr: Union[Value, AssignmentTarget]) -> Value:
         """A helper to get collection length, used by several subclasses."""
-        return self.builder.builder.builtin_call(
-            [self.builder.read(expr, self.line)],
-            'builtins.len',
-            self.line,
-        )
+        return self.builder.builder.builtin_len(self.builder.read(expr, self.line), self.line)
 
 
 class ForIterable(ForGenerator):
@@ -352,7 +347,7 @@ class ForIterable(ForGenerator):
         # for the for-loop. If we are inside of a generator function, spill these into the
         # environment class.
         builder = self.builder
-        iter_reg = builder.primitive_op(iter_op, [expr_reg], self.line)
+        iter_reg = builder.call_c(iter_op, [expr_reg], self.line)
         builder.maybe_spill(expr_reg)
         self.iter_target = builder.maybe_spill(iter_reg)
         self.target_type = target_type
@@ -364,7 +359,7 @@ class ForIterable(ForGenerator):
         # for NULL (an exception does not necessarily have to be raised).
         builder = self.builder
         line = self.line
-        self.next_reg = builder.primitive_op(next_op, [builder.read(self.iter_target, line)], line)
+        self.next_reg = builder.call_c(next_op, [builder.read(self.iter_target, line)], line)
         builder.add(Branch(self.next_reg, self.loop_exit, self.body_block, Branch.IS_ERROR))
 
     def begin_body(self) -> None:
@@ -386,7 +381,7 @@ class ForIterable(ForGenerator):
         # an exception was raised during the loop, then err_reg wil be set to
         # True. If no_err_occurred_op returns False, then the exception will be
         # propagated using the ERR_FALSE flag.
-        self.builder.primitive_op(no_err_occurred_op, [], self.line)
+        self.builder.call_c(no_err_occurred_op, [], self.line)
 
 
 def unsafe_index(
@@ -465,10 +460,10 @@ class ForSequence(ForGenerator):
         builder = self.builder
         line = self.line
         step = 1 if not self.reverse else -1
-        builder.assign(self.index_target, builder.primitive_op(
-            unsafe_short_add,
-            [builder.read(self.index_target, line),
-             builder.add(LoadInt(step))], line), line)
+        add = builder.binary_int_op(short_int_rprimitive,
+                                    builder.read(self.index_target, line),
+                                    builder.add(LoadInt(step)), BinaryIntOp.ADD, line)
+        builder.assign(self.index_target, add, line)
 
 
 class ForDictionaryCommon(ForGenerator):
@@ -486,8 +481,8 @@ class ForDictionaryCommon(ForGenerator):
     since they may override some iteration methods in subtly incompatible manner.
     The fallback logic is implemented in CPy.h via dynamic type check.
     """
-    dict_next_op = None  # type: ClassVar[OpDescription]
-    dict_iter_op = None  # type: ClassVar[OpDescription]
+    dict_next_op = None  # type: ClassVar[CFunctionDescription]
+    dict_iter_op = None  # type: ClassVar[CFunctionDescription]
 
     def need_cleanup(self) -> bool:
         # Technically, a dict subclass can raise an unrelated exception
@@ -505,14 +500,14 @@ class ForDictionaryCommon(ForGenerator):
         self.size = builder.maybe_spill(self.load_len(self.expr_target))
 
         # For dict class (not a subclass) this is the dictionary itself.
-        iter_reg = builder.primitive_op(self.dict_iter_op, [expr_reg], self.line)
+        iter_reg = builder.call_c(self.dict_iter_op, [expr_reg], self.line)
         self.iter_target = builder.maybe_spill(iter_reg)
 
     def gen_condition(self) -> None:
         """Get next key/value pair, set new offset, and check if we should continue."""
         builder = self.builder
         line = self.line
-        self.next_tuple = self.builder.primitive_op(
+        self.next_tuple = self.builder.call_c(
             self.dict_next_op, [builder.read(self.iter_target, line),
                                 builder.read(self.offset_target, line)], line)
 
@@ -533,13 +528,13 @@ class ForDictionaryCommon(ForGenerator):
         builder = self.builder
         line = self.line
         # Technically, we don't need a new primitive for this, but it is simpler.
-        builder.primitive_op(dict_check_size_op,
-                             [builder.read(self.expr_target, line),
-                              builder.read(self.size, line)], line)
+        builder.call_c(dict_check_size_op,
+                       [builder.read(self.expr_target, line),
+                        builder.read(self.size, line)], line)
 
     def gen_cleanup(self) -> None:
         # Same as for generic ForIterable.
-        self.builder.primitive_op(no_err_occurred_op, [], self.line)
+        self.builder.call_c(no_err_occurred_op, [], self.line)
 
 
 class ForDictionaryKeys(ForDictionaryCommon):
@@ -635,9 +630,9 @@ class ForRange(ForGenerator):
         # short ints.
         if (is_short_int_rprimitive(self.start_reg.type)
                 and is_short_int_rprimitive(self.end_reg.type)):
-            new_val = builder.primitive_op(
-                unsafe_short_add, [builder.read(self.index_reg, line),
-                                   builder.add(LoadInt(self.step))], line)
+            new_val = builder.binary_int_op(short_int_rprimitive,
+                            builder.read(self.index_reg, line),
+                            builder.add(LoadInt(self.step)), BinaryIntOp.ADD, line)
 
         else:
             new_val = builder.binary_op(
@@ -665,9 +660,9 @@ class ForInfiniteCounter(ForGenerator):
         # We can safely assume that the integer is short, since we are not going to wrap
         # around a 63-bit integer.
         # NOTE: This would be questionable if short ints could be 32 bits.
-        new_val = builder.primitive_op(
-            unsafe_short_add, [builder.read(self.index_reg, line),
-                               builder.add(LoadInt(1))], line)
+        new_val = builder.binary_int_op(short_int_rprimitive,
+                builder.read(self.index_reg, line),
+                builder.add(LoadInt(1)), BinaryIntOp.ADD, line)
         builder.assign(self.index_reg, new_val, line)
         builder.assign(self.index_target, new_val, line)
 
